/*  
file: build_lattice.hpp 
Header file for lattice_graph contruction

Spinner v.2, a Monte Carlo program for the simulation of 'classical'
magnetic systems.

Written by Mark de Vries, markadevries@gmail.com

Copyright (c), Aug 2005-Jan 2007, Dec 2012, The University Court of the University of
Edinburgh, Old College, Edinburgh, UK  

Copyright (c), Aug 2005-Oct 2006, Council for the Central Laboratory of the 
Research  Councils, Rutherford Appleton Laboratory, Chilton, Didcot, Oxfordshire, UK

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef BUILD_LATTICE
#define BUILD_LATTICE

#include <boost/tokenizer.hpp>
#include <boost/format.hpp>
#include <boost/random/lagged_fibonacci.hpp> 
#include <boost/random/variate_generator.hpp>
#include <boost/random/uniform_real.hpp>


#include "lattice_graph.hpp"
#include "options.hpp"

using namespace boost;
using namespace boost::numeric;

struct atom_data {
  std::string label;
  euclid_vector x; //position in unit cell measured in fractions of a,b,c
  double s,g,f; //spin magnitude and g factor.
  int number_of_bonds;
}; 

struct bond_data {
  double J, length;
  std::string site_i,site_j;
};
 
std::ostream& operator<<(std::ostream& os, const atom_data& v);
/*{
  os<<'('<<v.x[0]<<','<<v.x[1]<<','<<v.x[2]<<"),"<<v.s;
  return os;
}*/

std::vector<atom_data> parse_atoms(const std::vector<std::string>& values);
std::vector<bond_data> parse_bonds(const std::vector<std::string>& values);

lattice_graph build_lattice(po::variables_map& vm);
lattice_graph load_lattice(po::variables_map& vm);

#endif
