/*  
Spinner v.2.2

A Monte Carlo program for the simulation of 'classical'
magnetic systems.

Written by Mark de Vries, markadevries@gmail.com

Copyright (c), Aug 2005-Jan 2007, Dec 2012, The University Court of the University of
Edinburgh, Old College, Edinburgh, UK  

Copyright (c), Aug 2005-Oct 2006, Council for the Central Laboratory of the 
Research  Councils, Rutherford Appleton Laboratory, Chilton, Didcot, Oxfordshire, UK

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "options.hpp"
#include "build_lattice.hpp"
#include "flippers.hpp"
#include "hamiltonians.hpp"
#include "metropolis.hpp"
#include "data_handler.hpp"

using namespace boost;

int main(int ac ,char* av[] )
{
  try 
    {

    po::variables_map variables(parse_options(ac,av,""));
    show_title();
  
    lattice_graph lattice(build_lattice(variables));

    show_banner_simulation();

    map_of_fifos data;
    data_handler dh(variables,data,lattice);

    std::string Hamiltonian = variables["Hamiltonian"].as<std::string>();
    int spin_dimension = variables["spin-dimension"].as<int>();
    
    if (Hamiltonian=="super-exchange") {
      if (spin_dimension == 1) {
	Ising flipper;
        metropolis SimulationMethod(variables,data,lattice);
	super_exchange hamiltonian(variables);
	dh.run(SimulationMethod,hamiltonian,lattice,flipper);
      }
      else if (spin_dimension == 2) {
	XY flipper;
        metropolis SimulationMethod(variables,data,lattice);
	super_exchange hamiltonian(variables);
	dh.run(SimulationMethod,hamiltonian,lattice,flipper);
      }
      else if (spin_dimension == 3) {
	Heisenberg flipper;
	metropolis SimulationMethod(variables,data,lattice);
	super_exchange hamiltonian(variables);
	dh.run(SimulationMethod,hamiltonian,lattice,flipper);
      }
      else 
	throw po::invalid_option_value("Spin dimension can only be 0,1,2 or 3");
    }
    else if (Hamiltonian == "Molecular Field") {
      flipper_base flipper; //does not do anything here, but needs to be declared.
      metropolis_heisenberg SimulationMethod(variables,data,lattice);
      MolecularField hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }      
    else if (Hamiltonian == "Heisenberg") {
      flipper_base flipper; //does not do anything here, but needs to be declared.
      metropolis_heisenberg SimulationMethod(variables,data,lattice);
      super_exchange hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }      
    else  if (Hamiltonian == "Potts") {
      Potts flipper;
      metropolis SimulationMethod(variables,data,lattice);
      H_Potts hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    else if (Hamiltonian=="DMI"){
      flipper_base flipper;
      metropolis_heisenberg SimulationMethod(variables,data,lattice);
      DMI hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    else if (Hamiltonian=="local MAE") {
      flipper_base flipper;
      metropolis_heisenberg SimulationMethod(variables,data,lattice);
      local_MAE hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    else if (Hamiltonian=="dipole-dipole reaction field" ) {
      flipper_base flipper;
      metropolis_heisenberg SimulationMethod(variables,data,lattice);    
      dipole_reaction_field hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    else if (Hamiltonian=="dipole-dipole truncated" ) {
      flipper_base flipper;
      metropolis_heisenberg SimulationMethod(variables,data,lattice);    
      dipole_truncated hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    //Disabled for now. It may work, but that cannot be tested at the moment
    //because it is too slow. Need to implement a dynamic database of solutions
    else if (Hamiltonian=="dipole-dipole Ewald sum" ) {
      Heisenberg flipper;
      metropolis_heisenberg SimulationMethod(variables,data,lattice);    
      dipole_Ewald_sum hamiltonian(variables);
      dh.run(SimulationMethod,hamiltonian,lattice,flipper);
    }
    
    //Use this one only if the dipole interaction is only secondary.
    else 
      throw po::invalid_option_value("The specified Hamiltonian is not recogised.");
    }

  catch(std::exception& er) 
    {
      std::cerr<<er.what() << "\n";
      return 1;
    }  

  return 0;
}
